#include "stdafx.h"
#include "kdTree.h"

struct sortX {
	bool operator()( Primitive *p1,  Primitive *p2) const {
		//	float min = p1->p1[0];
		////	min = (p1->p2[0] < min) ? p1->p2[0] : min;
		////	min = (p1->p3[0] < min) ? p1->p3[0] : min;

		return (p1->minX() < p2->minX());
	}
};

struct sortY {
	bool operator()( Primitive *p1,  Primitive *p2) const {
		//float min = p1->p1[1];
		//min = (p1->p2[1] < min) ? p1->p2[1] : min;
		//min = (p1->p3[1] < min) ? p1->p3[1] : min;

		return p1->minY() < p2->minY();
	}
};

struct sortZ {
	bool operator()( Primitive *p1,  Primitive *p2) const {
		//float min = p1->p1[2];
		//min = (p1->p2[2] < min) ? p1->p2[2] : min;
		//min = (p1->p3[2] < min) ? p1->p3[2] : min;

		return p1->minZ() < p2->minZ();
	}
};

kdTree::kdTree(vector<Primitive*> prim) {
	copy(prim.begin(), prim.end(), back_inserter(objprim));

	kdT = create_kdTree(0, objprim);
}

kdTree::~kdTree() {
	for(vector<Primitive*>::iterator iter = objprim.begin();
			iter != objprim.end(); iter++) {
		Primitive *prim = *iter;

		delete prim;
	}
}

_kdNode* kdTree::create_kdTree(int dir, vector<Primitive *> triangles) {
	_kdNode *node = new _kdNode();

	if (triangles.size() < 5) {
		//		node->prim.swap(sorted_x);
		node->left  = NULL;
		node->right = NULL;
	}
	else {
		if (dir == 0) {
			sort(triangles.begin(), triangles.end(), sortX());
		}
		else if (dir == 1) {
			sort(triangles.begin(), triangles.end(), sortY());
		}
		else if (dir == 2) {
			sort(triangles.begin(), triangles.end(), sortZ());
		}

		int size = triangles.size() / 2;

		vector<Primitive *> sorted_A(triangles.begin(), triangles.begin() + size);
		vector<Primitive *> sorted_B(triangles.begin() + size, triangles.end());

		node->left  = create_kdTree((dir +1) % 3, sorted_A);
		node->right = create_kdTree((dir +1) % 3, sorted_B);
	}

	/** Calculate Bounding box */
	if (node->left != NULL && node->right != NULL) {
		node->xmin = (node->left->xmin < node->right->xmin) ? node->left->xmin : node->right->xmin;
		node->xmax = (node->left->xmax > node->right->xmax) ? node->left->xmax : node->right->xmax;

		node->ymin = (node->left->ymin < node->right->ymin) ? node->left->ymin : node->right->ymin;
		node->ymax = (node->left->ymax > node->right->ymax) ? node->left->ymax : node->right->ymax;

		node->zmin = (node->left->zmin < node->right->zmin) ? node->left->zmin : node->right->zmin;
		node->zmax = (node->left->zmax > node->right->zmax) ? node->left->zmax : node->right->zmax;
	}
	else {
		/** no luck, do it yourself */
		float xmin = INT_MAX;
		float xmax = INT_MIN;

		float ymin = INT_MAX;
		float ymax = INT_MIN;

		float zmin = INT_MAX;
		float zmax = INT_MIN;

		for(vector<Primitive *>::iterator iter = triangles.begin(); iter != triangles.end(); ++iter) {
			Primitive *tri = *iter;

			if (tri->minX() < xmin)				xmin = tri->minX();
			if (tri->maxX() > xmax)				xmax = tri->maxX();

			if (tri->minY() < ymin)				ymin = tri->minY();
			if (tri->maxY() > ymax)				ymax = tri->maxY();

			if (tri->minZ() < zmin)				zmin = tri->minZ();
			if (tri->maxZ() > xmax)				zmax = tri->maxZ();
		}

		copy(triangles.begin(), triangles.end(), back_inserter(node->prim));

		node->xmin = xmin;		node->xmax = xmax;
		node->ymin = ymin;		node->ymax = ymax;
		node->zmin = zmin;		node->zmax = zmax;
	}

	// Plane equation
	// ax + bx + cx + d = 0
	//   
	//     | a |
	// n = | b |
	//     | c |

	node->n1[0] = 1.0f;
	node->n1[1] = 0.0f;
	node->n1[2] = 0.0f;
	node->d1 = -node->n1[0] * node->xmax - node->n1[1] * node->ymin - node->n1[2] * node->zmin;

	node->n2[0] =-1.0f;
	node->n2[1] = 0.0f;
	node->n2[2] = 0.0f;
	node->d2 = -node->n2[0] * node->xmin - node->n2[1] * node->ymin - node->n2[2] * node->zmin;

	node->n3[0] = 0.0f;
	node->n3[1] = 1.0f;
	node->n3[2] = 0.0f;
	node->d3 = -node->n3[0] * node->xmin - node->n3[1] * node->ymax - node->n3[2] * node->zmin;

	node->n4[0] = 0.0f;
	node->n4[1] =-1.0f;
	node->n4[2] = 0.0f;
	node->d4 = -node->n4[0] * node->xmin - node->n4[1] * node->ymin - node->n4[2] * node->zmin;

	node->n5[0] = 0.0f;
	node->n5[1] = 0.0f;
	node->n5[2] = 1.0f;
	node->d5 = -node->n5[0] * node->xmin - node->n5[1] * node->ymin - node->n5[2] * node->zmax;

	node->n6[0] = 0.0f;
	node->n6[1] = 0.0f;
	node->n6[2] =-1.0f;
	node->d6 = -node->n6[0] * node->xmin - node->n6[1] * node->ymin - node->n6[2] * node->zmin;

	//printf("%.2f-%.2f : %.2f-%.2f : %.2f-%.2f\n", node->xmin, node->xmax, node->ymin, node->ymax, node->zmin, node->zmax);

	return node;
}


	// virtual functions (must be declared in descendend class)
	 float kdTree::t(float *p,float *d) {
	 return t_recursive(p, d, kdT, 10000);
	 }

	 float kdTree::tex(float *p,float *d, float *uv) {
	 return tex_recursive(p, d, uv, kdT, 10000);
	 }

 
void kdTree::transform(float m[3][3]) {
	for(vector<Primitive *>::iterator iter = objprim.begin();
		iter != objprim.end(); ++iter) {
			((Primitive*)*iter)->transform(m);
	}

	kdT = create_kdTree(0, objprim);
}

void kdTree::translate(float *t) {
	for(vector<Primitive *>::iterator iter = objprim.begin();
		iter != objprim.end(); ++iter) {
			((Primitive*)*iter)->translate(t);
	}

	kdT = create_kdTree(0, objprim);
}

/**
* cut a vector with a plan
*
* @args p start of vector
* @args d direction of vector
* @args n normal of plane
* @args dx parameter d of plane
* @args min1, max1, min2, max2 border of plane (checks)
*/
void kdTree::cut_plane(float *p, float *d, float *n, float dx, float *cut) {
	float dividend = - (dot(n, p) + dx);
	float divisor  = dot(n, d);
	float t = dividend / divisor;

	add(p, scale(t, d, cut), cut);
}

float kdTree::t_recursive(float *p, float *d, _kdNode *node, float min) {
	if (node->left != NULL) {
		float cut[3];

		// Plane 1 of 6 (y-z-plane)
		cut_plane(p, d, node->n1, node->d1, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[1] >= node->ymin && cut[1] <= node->ymax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = t_recursive(p, d, node->left, min);
				min = t_recursive(p, d, node->right, min);

				return min;
		}

		// Plane 2 of 6 (y-z-plane 2)
		cut_plane(p, d, node->n2, node->d2, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[1] >= node->ymin && cut[1] <= node->ymax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = t_recursive(p, d, node->left, min);
				min = t_recursive(p, d, node->right, min);

				return min;
		}

		// Plane 3 of 6 (x-z-plane)
		cut_plane(p, d, node->n3, node->d3, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = t_recursive(p, d, node->left, min);
				min = t_recursive(p, d, node->right, min);

				return min;
		}

		// Plane 4 of 6 (x-z-plane 2)
		cut_plane(p, d, node->n4, node->d4, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = t_recursive(p, d, node->left, min);
				min = t_recursive(p, d, node->right, min);

				return min;
		}

		// Plane 5 of 6 (x-y-plane)
		cut_plane(p, d, node->n5, node->d5, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[1] >= node->ymin && cut[1] <= node->ymax) {
				min = t_recursive(p, d, node->left, min);
				min = t_recursive(p, d, node->right, min);

				return min;
		}

		// Plane 6 of 6 (x-y-plane)
		cut_plane(p, d, node->n6, node->d6, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[1] >= node->ymin && cut[1] <= node->ymax) {
				min = t_recursive(p, d, node->left, min);
				min = t_recursive(p, d, node->right, min);

				return min;
		}
	}
	else {
		for(vector<Primitive *>::iterator iter = node->prim.begin(); iter != node->prim.end(); ++iter) {
			float tmp_min = ((Primitive*)*iter)->t(p, d);

			if (tmp_min < min) {
				min = tmp_min;
				cut_prim = *iter;
			}
		}
	}

	return min;
}

float kdTree::tex_recursive(float *p, float *d, float *uv, _kdNode *node, float min) {
	if (node->left != NULL) {
		float cut[3];

		// Plane 1 of 6 (y-z-plane)
		cut_plane(p, d, node->n1, node->d1, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[1] >= node->ymin && cut[1] <= node->ymax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = tex_recursive(p, d, uv, node->left, min);
				min = tex_recursive(p, d, uv, node->right, min);

				return min;
		}

		// Plane 2 of 6 (y-z-plane 2)
		cut_plane(p, d, node->n2, node->d2, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[1] >= node->ymin && cut[1] <= node->ymax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = tex_recursive(p, d, uv, node->left, min);
				min = tex_recursive(p, d, uv, node->right, min);

				return min;
		}

		// Plane 3 of 6 (x-z-plane)
		cut_plane(p, d, node->n3, node->d3, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = tex_recursive(p, d, uv, node->left, min);
				min = tex_recursive(p, d, uv, node->right, min);

				return min;
		}

		// Plane 4 of 6 (x-z-plane 2)
		cut_plane(p, d, node->n4, node->d4, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[2] >= node->zmin && cut[2] <= node->zmax) {
				min = tex_recursive(p, d, uv, node->left, min);
				min = tex_recursive(p, d, uv, node->right, min);

				return min;
		}

		// Plane 5 of 6 (x-y-plane)
		cut_plane(p, d, node->n5, node->d5, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[1] >= node->ymin && cut[1] <= node->ymax) {
				min = tex_recursive(p, d, uv, node->left, min);
				min = tex_recursive(p, d, uv, node->right, min);

				return min;
		}

		// Plane 6 of 6 (x-y-plane)
		cut_plane(p, d, node->n6, node->d6, cut);

		// now we simply have to check if cut is inside the plane
		if (cut[0] >= node->xmin && cut[0] <= node->xmax &&
			cut[1] >= node->ymin && cut[1] <= node->ymax) {
				min = tex_recursive(p, d, uv, node->left, min);
				min = tex_recursive(p, d, uv, node->right, min);

				return min;
		}
	}
	else {
		for(vector<Primitive *>::iterator iter = node->prim.begin(); iter != node->prim.end(); ++iter) {
			float uvmin[3];
			float tmp_min = ((Primitive*)*iter)->tex(p, d, uvmin);

			if (tmp_min < min) {
				min = tmp_min;
				cut_prim = *iter;

				// get uvmin
				uv[0] = uvmin[0];
				uv[1] = uvmin[1];
			}
		}
	}

	return min;
}

// position
float kdTree::minX() {
	if (kdT != NULL)
		return kdT->xmin;
	else
		return -1024;
}

float kdTree::maxX() {
	if (kdT != NULL)
		return kdT->xmax;
	else
		return 1024;
}

float kdTree::minY() {
	if (kdT != NULL)
		return kdT->ymin;
	else
		return -1024;
}

float kdTree::maxY() {
	if (kdT != NULL)
		return kdT->ymax;
	else
		return 1024;
}

float kdTree::minZ() {
	if (kdT != NULL)
		return kdT->zmin;
	else
		return -1024;

}

float kdTree::maxZ() {
	if (kdT != NULL)
		return kdT->zmax;
	else
		return 1024;
}
