#pragma once
#include <vector>
#include "primitive.h"

using namespace std;

typedef struct _kdNode {
	_kdNode *left;
	_kdNode *right;

	vector<Primitive *> prim;

	float xmin, xmax;
	float ymin, ymax;
	float zmin, zmax;

	float n1[3]; float d1;
	float n2[3]; float d2;
	float n3[3]; float d3;
	float n4[3]; float d4;
	float n5[3]; float d5;
	float n6[3]; float d6;
};

class kdTree : public Primitive {

public:
	kdTree(vector<Primitive*> prim);
	_kdNode* create_kdTree(int dir, vector<Primitive *> triangles);
	~kdTree();

	// virtual functions (must be declared in descendend class)
	virtual float t(float *p,float *d);
	virtual float tex(float *p,float *d, float *uv);
	virtual void transform(float m[3][3]);
	virtual void translate(float *t);

	virtual float cut_dist(float *p,float *d)				{ return cut_prim->cut_dist(p, d); };
	virtual void normal(float *hit, float *n)				{ cut_prim->normal(hit, n); };
	virtual void partial_derivatives(float *uv, float *duv) { cut_prim->partial_derivatives(uv, duv); };
	virtual void material(float *m)							{ cut_prim->material(m); };
	virtual float get_kd(void)								{ return cut_prim->get_kd(); };
	virtual float get_ks(void)								{ return cut_prim->get_ks(); };
	virtual float get_kr(void)								{ return cut_prim->get_kr(); };
	virtual float get_index(void)							{ return cut_prim->get_index(); };
	virtual int get_texture_index(void)						{ return cut_prim->get_texture_index(); };
	virtual int get_bump_index(void)						{ return cut_prim->get_bump_index(); };
	virtual bool textured(void)								{ return false; };
	virtual bool bump_mapping(void)							{ return false; };

	// position
	virtual float minX();
	virtual float maxX();
	virtual float minY();
	virtual float maxY();
	virtual float minZ();
	virtual float maxZ();

private:
	float t_recursive(float *p, float *d, _kdNode *node, float min);
	float tex_recursive(float *p, float *d, float *uv, _kdNode *node, float min);
	void cut_plane(float *p, float *d, float *n, float dx, float *cut);

private:
	Primitive *cut_prim;
	_kdNode   *kdT;

	vector<Primitive *> objprim;

};