#pragma once
#include <math.h>

class Primitive {

public:
	static inline float dot(float a[], float b[]) {
		return a[0]*b[0] + a[1]*b[1] + a[2]*b[2];
	}

	static inline void crossp(float a[], float b[], float c[]) {
		c[0] = a[1]*b[2] - a[2]*b[1];
		c[1] = a[2]*b[0] - a[0]*b[2];
		c[2] = a[0]*b[1] - a[1]*b[0];
	}

	static inline void norm(float v[]) {
		float norm = sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);

		v[0] = v[0]/norm;
		v[1] = v[1]/norm;
		v[2] = v[2]/norm;
	}

	static inline float* add(float *a, float *b, float *res) {
		res[0] = a[0] + b[0];
		res[1] = a[1] + b[1];
		res[2] = a[2] + b[2];

		return res;
	}

	static inline float* scale(float a, float *b, float *res) {
		res[0] = a * b[0];
		res[1] = a * b[1];
		res[2] = a * b[2];

		return res;
	}

	inline void rotate(float degree, float x, float y, float z) {
		if (degree == 0)
			return;

		float angle = degree/180.0f * 3.14159265358979;
		float m[3][3];

		m[0][0] = x*x + cosf(angle) * (1-x*x);
		m[1][0] = x*y + (1 - cosf(angle)) + z*sinf(angle);
		m[2][0] = x*z + (1 - cosf(angle)) - y*sinf(angle);

		m[0][1] = x*y + (1 - cosf(angle)) - z*sinf(angle);
		m[1][1] = y*y + cosf(angle) * (1-y*y);
		m[2][1] = y*z * (1 - cosf(angle)) + x * sinf(angle);

		m[0][2] = x*z + (1 - cosf(angle)) + y*sinf(angle);
		m[1][2] = y*z * (1 - cosf(angle)) - x * sinf(angle);
		m[2][2] = z*z + cosf(angle) * (1-z*z);

		transform(m);

		return;
	}

	// virtual functions (must be declared in descendend class)
	virtual float t(float *p,float *d) = 0;
	virtual float tex(float *p,float *d, float *uv) = 0;
	virtual float cut_dist(float *p,float *d) = 0; 
	virtual void transform(float m[3][3]) = 0;
	virtual void translate(float *t) = 0;
	virtual void normal(float *hit, float *n) = 0;
	virtual void partial_derivatives(float *uv, float *duv) = 0;
	virtual void material(float *m) = 0;
	virtual float get_kd(void) = 0;
	virtual float get_ks(void) = 0;
	virtual float get_kr(void) = 0;
	virtual float get_index(void) = 0;
	virtual int get_texture_index(void) = 0;
	virtual int get_bump_index(void) = 0;
	virtual bool textured(void) = 0;
	virtual bool bump_mapping(void) = 0;

	// position
	virtual float minX() = 0;
	virtual float maxX() = 0;
	virtual float minY() = 0;
	virtual float maxY() = 0;
	virtual float minZ() = 0;
	virtual float maxZ() = 0;
};