#pragma once

// Define Interface Data Types / Structures
//
typedef enum {
	raytxUnknown	= 0,	// images
	raytxBmp		= 1,
	raytxJpg		= 2,
	raytxPng		= 3,
	raytxTga		= 4,
	raytxGif		= 5,
	raytxIco		= 6,
	raytxEmf		= 7,
	raytxWmf		= 8,
	// add new ones at the end
} rayeglTexType;

typedef enum {
	raytxNoFilter	= 0,
	raytxBilinear	= 1,
	raytxTrilinear	= 2,
	// add new ones at the end
} rayeglTexFilterType;

typedef	struct
{
	unsigned int		TextureID;									// Texture ID Used To Select A Texture
	rayeglTexType		TexType;									// Texture Format
	unsigned int		Width;										// Image Width
	unsigned int		Height;										// Image Height
	unsigned int		Type;										// Image Type (GL_RGB, GL_RGBA)
	unsigned int		Bpp;										// Image Color Depth In Bits Per Pixel
	unsigned char       *rawTexture;
} rayTexture;

typedef struct {
	float	s;
	float	t;
} _rayglTexturCord;


typedef struct {
	_rayglTexturCord TopRight;
	_rayglTexturCord TopLeft;
	_rayglTexturCord BottomLeft;
	_rayglTexturCord BottomRight;
} rayglTexturCordTable;

// Define Private Structurs
//

// Header
typedef struct {
	unsigned char ImgIdent;
	unsigned char ignored[ 1 ];
	unsigned char ImgType;
	unsigned char ignored2[ 9 ];
	unsigned char WidthLo;
	unsigned char WidthHi;
	unsigned char HeightLo;
	unsigned char HeightHi;
	unsigned char Bpp;
	unsigned char ignored3[ 1 ];
} _rayTGAHeader;

#define rayRGB                            0x1907
#define rayRGBA                           0x1908

// define TextureLoader Class
//
class rayTextureLoader {
public:
	// methods
	rayTextureLoader();
	virtual				~rayTextureLoader();
	void				SetAlphaMatch(bool fEnabled, unsigned char RedAlphaMatch, unsigned char GreenAlphaMatch, unsigned char BlueAlphaMatch);
	void				SetHighQualityTextures(bool fEnabled);
	void				SetMipMapping(bool fEnabled);
	void				SetTextureFilter(rayeglTexFilterType type);

	int					LoadTextureFromDisk(char *szFileName, rayTexture *prayTexture);
	int					LoadTextureFromRam(unsigned char *pData, int Length, rayTexture *prayTexture, rayeglTexType TexType);
	int					LoadTextureFromResource(unsigned int ResourceName, char *pResourceType, rayTexture *prayTexture, rayeglTexType TexType);
	void				FreeTexture(rayTexture *prayTexture);
	// variables

private:
	// methods
	int					BuildTexture(char *szPathName, rayTexture *prayTexture);
	int					LoadTGAFromDisk(char *pszFileName, rayTexture *prayTexture);

	int					LoadJPG_GIFResource(char *pResourceName, char *pResourceType, rayTexture *prayTexture);
	int					LoadTGAResource(char *pResourceName, char *pResourceType, rayTexture *prayTexture);

	int					GenerateTexture(rayTexture *prayTexture, unsigned char *pImgData);

	void				ExtensionFromFilename(char *szFileName, char *szExtension);

	// variables
	bool				m_fAlphaConversion;
	bool				m_fHighQualityTextures;
	bool				m_fMipMapping;
	rayeglTexFilterType	m_TextureFilterType;

	unsigned char		m_RedAlphaMatch;
	unsigned char		m_GreenAlphaMatch;
	unsigned char 		m_BlueAlphaMatch;
};
