#include "stdafx.h"
#include "sphere.h"


void Sphere::translate(float *t){
	c[0] += t[0];
	c[1] += t[1];
	c[2] += t[2];
}

void Sphere::transform(float m[3][3]){
	float c0 = c[0]; float c1 = c[1]; float c2 = c[2];
	c[0] = m[0][0]*c0+m[0][1]*c1+m[0][2]*c2;
	c[1] = m[1][0]*c0+m[1][1]*c1+m[1][2]*c2;
	c[2] = m[2][0]*c0+m[2][1]*c1+m[2][2]*c2;
	if (is_textured||has_bump_map){
		float u0 = up[0]; float u1 = up[1]; float u2 = up[2];
		up[0] = m[0][0]*u0+m[0][1]*u1+m[0][2]*u2;
		up[1] = m[1][0]*u0+m[1][1]*u1+m[1][2]*u2;
		up[2] = m[2][0]*u0+m[2][1]*u1+m[2][2]*u2;
		float r0 = right[0]; float r1 = right[1]; float r2 = right[2];
		right[0] = m[0][0]*r0+m[0][1]*r1+m[0][2]*r2;
		right[1] = m[1][0]*r0+m[1][1]*r1+m[1][2]*r2;
		right[2] = m[2][0]*r0+m[2][1]*r1+m[2][2]*r2;
	}
}

float Sphere::t(float *p,float *d){

	float t;
	float b = 2*((p[0]-c[0])*d[0] + (p[1]-c[1])*d[1] + (p[2]-c[2])*d[2]);
	float k = (p[0]-c[0])*(p[0]-c[0]) + (p[1]-c[1])*(p[1]-c[1]) + (p[2]-c[2])*(p[2]-c[2]) - r*r;

	if ((b*b-4*k)>=0){

		float t1 = (-b + sqrt(b*b-4*k))/(2.0);
		float t2 = (-b - sqrt(b*b-4*k))/(2.0);
		if ((t1>0)||(t2>0)){

			if (t1<0){
				t = t2;
			}else if(t2<0){
				t = t1;
			}else if(t1<t2){
				t = t1;
			}else{
				t = t2;
			}	
			return t;
		}
	}
	return 10000;
}

float Sphere::tex(float *p,float *d, float *uv){

	//x = sintheta sinphi
	//y = cosphi
	//z = costheta sinphi
	float t; float hit[3]; float phi; float theta; float cross[3];
	float b = 2*((p[0]-c[0])*d[0] + (p[1]-c[1])*d[1] + (p[2]-c[2])*d[2]);
	float k = (p[0]-c[0])*(p[0]-c[0]) + (p[1]-c[1])*(p[1]-c[1]) + (p[2]-c[2])*(p[2]-c[2]) - r*r;

	if ((b*b-4*k)>=0){

		float t1 = (-b + sqrt(b*b-4*k))/(2.0);
		float t2 = (-b - sqrt(b*b-4*k))/(2.0);
		if ((t1>0)||(t2>0)){

			if (t1<0){
				t = t2;
			}else if(t2<0){
				t = t1;
			}else if(t1<t2){
				t = t1;
			}else{
				t = t2;
			}	
			hit[0] = p[0] + t*d[0] - c[0];
			hit[1] = p[1] + t*d[1] - c[1];
			hit[2] = p[2] + t*d[2] - c[2];
			norm(hit);
			phi = acos(dot(hit,up));
			uv[1] = 1.0-phi/(3.14159265358979);
			if (sin(phi) != 0.0){
				theta = acos(dot(hit,right)/sin(phi))/(2.0*3.14159265358979);
				crossp(up,right,cross);
				if (dot(cross,hit) > 0.0){
					uv[0] = theta;
				}else{
					uv[0] = 1.0 - theta;
				}
			}else{
				
			}
			return t;
		}
	}
	return 10000;
}

bool Sphere::textured(){
	return is_textured;
}

bool Sphere::bump_mapping(){
	return has_bump_map;
}

int Sphere::get_texture_index(void){
	return tindex;
}

int Sphere::get_bump_index(void){
	return bindex;
}

float Sphere::cut_dist(float *p,float *d){

	float b = 2*((p[0]-c[0])*d[0] + (p[1]-c[1])*d[1] + (p[2]-c[2])*d[2]);
	float k = (p[0]-c[0])*(p[0]-c[0]) + (p[1]-c[1])*(p[1]-c[1]) + (p[2]-c[2])*(p[2]-c[2]) - r*r;

	if ((b*b-4*k)>=0){

		float t1 = (-b + sqrt(b*b-4*k))/(2.0);
		float t2 = (-b - sqrt(b*b-4*k))/(2.0);

		return abs(t1-t2);
	}
	return -1;
}


void Sphere::normal(float *hit, float *n){
	n[0] = hit[0]-c[0]; 
	n[1] = hit[1]-c[1];
	n[2] = hit[2]-c[2];
}

void Sphere::partial_derivatives(float *uv, float *duv){
	float pi = 3.14159265358979; float u = uv[0]; float v = uv[1];
	duv[0] = sin(u*pi*2.0)*cos(v*pi);
	duv[1] = -sin(v*pi);
	duv[2] = cos(u*pi*2.0)*cos(v*pi);
	duv[3] = cos(u*pi*2.0)*sin(v*pi);
	duv[4] = 0.0;
	duv[5] = -sin(u*pi*2.0)*sin(v*pi);
}

void Sphere::material(float *m){
	m[0] = mat[0];
	m[1] = mat[1];
	m[2] = mat[2];
}

float Sphere::get_kd(void){
	return kd;
}

float Sphere::get_ks(void){
	return ks;
}

float Sphere::get_kr(void){
	return kr;
}

float Sphere::get_index(void){
	return index;
}

Sphere::Sphere(float rad, float pos[3], float *m, float *k)
{
	r = rad;
	c[0] = pos[0];
	c[1] = pos[1];
	c[2] = pos[2];
	mat[0] = m[0];
	mat[1] = m[1];
	mat[2] = m[2];
	kd = k[0];
	ks = k[1];
	kr = k[2];
	index = k[3];
	is_textured = false;
	has_bump_map = false;
}

Sphere::Sphere(float rad, float pos[3], float *m, float *k, float *vup, float *vright, int tex, int btex)
{
	r = rad;
	c[0] = pos[0];
	c[1] = pos[1];
	c[2] = pos[2];
	mat[0] = m[0];
	mat[1] = m[1];
	mat[2] = m[2];
	kd = k[0];
	ks = k[1];
	kr = k[2];
	index = k[3];
	up[0] = vup[0];
	up[1] = vup[1];
	up[2] = vup[2];
	norm(up);
	right[0] = vright[0];
	right[1] = vright[1];
	right[2] = vright[2];
	norm(right);
	tindex = tex;
	bindex = btex;
	if (tex == -1){
		is_textured = false;
	}else{
		is_textured = true;
	}
	if (btex == -1){
		has_bump_map = false;
	}else{
		has_bump_map = true;
	}
}

Sphere::~Sphere(void)
{
}

// position
float Sphere::minX() {
	return c[0] - r;
}

float Sphere::maxX() {
	return c[0] + r;
}

float Sphere::minY() {
	return c[1] - r;
}

float Sphere::maxY() {
	return c[1] + r;
}

float Sphere::minZ() {
	return c[2] - r;
}

float Sphere::maxZ() {
	return c[2] + r;
}
