#include "StdAfx.h"
#include ".\obb.h"

float OBB::dotp(float *a, float *b){
	return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}

void OBB::translate(Vector3d t){
	center[0] = t[0];
	center[1] = t[1];
	center[2] = t[2];
}

void OBB::transform(float m[3][3]){
	float c0,c1,c2;
	c0 = waxisq[0]; c1 = waxisq[1]; c2 = waxisq[2];
	waxis[0] = m[0][0]*c0+m[0][1]*c1+m[0][2]*c2;
	waxis[1] = m[1][0]*c0+m[1][1]*c1+m[1][2]*c2;
	waxis[2] = m[2][0]*c0+m[2][1]*c1+m[2][2]*c2;	
	c0 = haxisq[0]; c1 = haxisq[1]; c2 = haxisq[2];
	haxis[0] = m[0][0]*c0+m[0][1]*c1+m[0][2]*c2;
	haxis[1] = m[1][0]*c0+m[1][1]*c1+m[1][2]*c2;
	haxis[2] = m[2][0]*c0+m[2][1]*c1+m[2][2]*c2;
	c0 = daxisq[0]; c1 = daxisq[1]; c2 = daxisq[2];
	daxis[0] = m[0][0]*c0+m[0][1]*c1+m[0][2]*c2;
	daxis[1] = m[1][0]*c0+m[1][1]*c1+m[1][2]*c2;
	daxis[2] = m[2][0]*c0+m[2][1]*c1+m[2][2]*c2;
}

bool OBB::overlap(OBB *other){
	int i;
	float t[3]; t[0] = center[0]-other->center[0]; t[1] = center[1]-other->center[1]; t[2] = center[2]-other->center[2];
	float *paxis[6]; float pa; float pb;
	paxis[0] = waxis;
	paxis[1] = haxis;
	paxis[2] = daxis;
	paxis[3] = other->waxis;
	paxis[4] = other->haxis;
	paxis[5] = other->daxis;
	for(i = 0; i < 6; i++){
		pa = width*abs(dotp(paxis[i],waxis)) + height*abs(dotp(paxis[i],haxis)) + depth*abs(dotp(paxis[i],daxis));
		pb = other->width*abs(dotp(paxis[i],other->waxis)) + other->height*abs(dotp(paxis[i],other->haxis)) 
		+ other->depth*abs(dotp(paxis[i],other->daxis));
		if(abs(dotp(paxis[i],t)) > (pa + pb)){
			return false;
		}
	}
	return true;
}

void OBB::set(float *size, float *w, float *h, float *d, float *c)
{
	width = size[0]; height = size[1]; depth = size[2];
	waxis[0] = w[0]; waxis[1] = w[1]; waxis[2] = w[2];
	haxis[0] = h[0]; haxis[1] = h[1]; haxis[2] = h[2];
	daxis[0] = d[0]; daxis[1] = d[1]; daxis[2] = d[2];
	waxisq[0] = w[0]; waxisq[1] = w[1]; waxisq[2] = w[2];
	haxisq[0] = h[0]; haxisq[1] = h[1]; haxisq[2] = h[2];
	daxisq[0] = d[0]; daxisq[1] = d[1]; daxisq[2] = d[2];
	center[0] = c[0]; center[1] = c[1]; center[2] = c[2];
}

OBB::OBB(float *size, float *w, float *h, float *d, float *c)
{
	width = size[0]; height = size[1]; depth = size[2];
	waxis[0] = w[0]; waxis[1] = w[1]; waxis[2] = w[2];
	haxis[0] = h[0]; haxis[1] = h[1]; haxis[2] = h[2];
	daxis[0] = d[0]; daxis[1] = d[1]; daxis[2] = d[2];
	waxisq[0] = w[0]; waxisq[1] = w[1]; waxisq[2] = w[2];
	haxisq[0] = h[0]; haxisq[1] = h[1]; haxisq[2] = h[2];
	daxisq[0] = d[0]; daxisq[1] = d[1]; daxisq[2] = d[2];
	center[0] = c[0]; center[1] = c[1]; center[2] = c[2];
}

OBB::OBB(void)
{
}

OBB::~OBB(void)
{
}
