#include "StdAfx.h"
#include ".\rigid_body.h"

float RIGID_BODY::dotp(float *a, float *b){
	return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}

void RIGID_BODY::crossp(float *a, float *b, float *c){
		c[0] = a[1]*b[2] - a[2]*b[1];
		c[1] = a[2]*b[0] - a[0]*b[2];
		c[2] = a[0]*b[1] - a[1]*b[0];
}

void RIGID_BODY::transpose(float m[3][3], float n[3][3]){
	n[0][0] = m[0][0]; n[0][1] = m[1][0]; n[0][2] = m[2][0];
	n[1][0] = m[0][1]; n[1][1] = m[1][1]; n[1][2] = m[2][1];
	n[2][0] = m[0][2]; n[2][1] = m[1][2]; n[2][2] = m[2][2];
}

void RIGID_BODY::matrix_add(float m[3][3], float n[3][3], float p[3][3]){
	p[0][0] = m[0][0] + n[0][0]; p[0][1] = m[0][1] + n[0][1]; p[0][2] = m[0][2] + n[0][2]; 
	p[1][0] = m[1][0] + n[1][0]; p[1][1] = m[1][1] + n[1][1]; p[1][2] = m[1][2] + n[1][2]; 
	p[2][0] = m[2][0] + n[2][0]; p[2][1] = m[2][1] + n[2][1]; p[2][2] = m[2][2] + n[2][2]; 
}

void RIGID_BODY::matrix_mult(float m[3][3], float n[3][3], float p[3][3]){
	float temp[3][3];
	transpose(n,temp);
	p[0][0] = dotp(m[0],n[0]); p[0][1] = dotp(m[0],n[1]); p[0][2] = dotp(m[0],n[2]); 
	p[1][0] = dotp(m[1],n[0]); p[1][1] = dotp(m[1],n[1]); p[1][2] = dotp(m[1],n[2]);
	p[2][0] = dotp(m[2],n[0]); p[2][1] = dotp(m[2],n[1]); p[2][2] = dotp(m[2],n[2]);
}

void RIGID_BODY::reorthonorm(float m[3][3]){
	float norm; float dot; 

	norm = sqrt(m[0][0]*m[0][0]+m[1][0]*m[1][0]+m[2][0]*m[2][0]);
	m[0][0] /= norm; m[1][0] /= norm; m[2][0] /= norm; 

	dot = m[0][0]*m[0][1]+m[1][0]*m[1][1]+m[2][0]*m[2][1];
	m[0][1] = m[0][1] - dot*m[0][0]; m[1][1] = m[1][1] - dot*m[1][0]; m[2][1] = m[2][1] - dot*m[2][0];
	norm = sqrt(m[0][1]*m[0][1]+m[1][1]*m[1][1]+m[2][1]*m[2][1]);
	m[0][1] /= norm; m[1][1] /= norm; m[2][1] /= norm; 
	
	m[0][2] = m[1][0]*m[2][1]-m[2][0]*m[1][1];
	m[1][2] = m[2][0]*m[0][1]-m[0][0]*m[2][1];
	m[2][2] = m[0][0]*m[1][1]-m[1][0]*m[0][1];
	norm = sqrt(m[0][2]*m[0][2]+m[1][2]*m[1][2]+m[2][2]*m[2][2]);
	m[0][2] /= norm; m[1][2] /= norm; m[2][2] /= norm; 
}

void RIGID_BODY::get_translation(float *trans){
	trans[0] = t[0]; trans[1] = t[1]; trans[2] = t[2]; 
}

void RIGID_BODY::get_transform(float MatrixA[3][3]){
	MatrixA[0][0] = A[0][0]; MatrixA[0][1] = A[0][1]; MatrixA[0][2] = A[0][2];
	MatrixA[1][0] = A[1][0]; MatrixA[1][1] = A[1][1]; MatrixA[1][2] = A[1][2];
	MatrixA[2][0] = A[2][0]; MatrixA[2][1] = A[2][1]; MatrixA[2][2] = A[2][2];
}

void RIGID_BODY::update_all_forces(Vector3d force){
	int i; 
	F.set(0.0,0.0,0.0); 
	torque.set(0.0,0.0,0.0);
	F = F + force;
	for (i = 0; i < number; i++){
		torque = torque + radii[i].CrossProduct(force);
	}
}

void RIGID_BODY::update_individual_forces(Vector3d *ind_forces){
	int i; 
	F.set(0.0,0.0,0.0); 
	torque.set(0.0,0.0,0.0);
	for (i = 0; i < number; i++){
		F = F + ind_forces[i];
		torque = torque + radii[i].CrossProduct(ind_forces[i]);
	}
}

void RIGID_BODY::update_step(float dt){

	float ws[3][3]; float temp1[3][3]; float temp2[3][3]; int i; 

	ws[0][0] = 0; ws[0][1] = -w[2]*dt; ws[0][2] = w[1]*dt; 
	ws[1][0] = w[2]*dt; ws[1][1] = 0; ws[1][2] = -w[0]*dt; 
	ws[2][0] = -w[1]*dt; ws[2][1] = w[0]*dt; ws[2][2] = 0; 
	
	xcm = xcm + vcm*dt; 
	t[0] = vcm[0]*dt; t[1] = vcm[1]*dt; t[2] = vcm[2]*dt;
	vcm = vcm + F*invM*dt; 
	matrix_mult(ws,A,temp1);
	matrix_add(temp1,A,A);
	reorthonorm(A);
	L = L + torque*dt;
	//I = A*Iq*A'
	transpose(A,temp1);
	matrix_mult(Iq,temp1,temp2);
	matrix_mult(A,temp2,I);
	//w = I*L
	w.set(Vector3d(I[0]).DotProduct(L),Vector3d(I[1]).DotProduct(L),Vector3d(I[2]).DotProduct(L));

	for (i = 0; i < number; i++){
		radii[i].set(Vector3d(A[0]).DotProduct(radiiq[i]),Vector3d(A[1]).DotProduct(radiiq[i]),Vector3d(A[2]).DotProduct(radiiq[i]));
		vertices[i] = xcm + radii[i];
		vels[i] = vcm + w.CrossProduct(radii[i]);
	}

	for (i = 0; i < nbr_of_normals; i++){
		normals[i].set(Vector3d(A[0]).DotProduct(normalsq[i]),Vector3d(A[1]).DotProduct(normalsq[i]),Vector3d(A[2]).DotProduct(normalsq[i]));
	}

	box->translate(xcm);
	box->transform(A);
}

RIGID_BODY::RIGID_BODY(Vector3d *verts, int nbr, face *fa, int fnbr,  Vector3d *norm, int nbr_of_norms, float invmass, Vector3d v, float MatrixA[3][3], Vector3d vecL, OBB *bbox, int ident)
{
	int i; float temp1[3][3]; float temp2[3][3];
	float det; float a = 0.0; float b = 0.0; float c = 0.0; float d = 0.0; float e = 0.0; 
	float f = 0.0; float g = 0.0; float h = 0.0; float j= 0.0;

	box = bbox; id = ident; faces = fa; nbr_of_faces = fnbr; normalsq = norm; nbr_of_normals = nbr_of_norms;
	nbr_of_verts = nbr;

	invM = invmass; number = nbr;
	vcm.set(v[0],v[1],v[2]);
	A[0][0] = MatrixA[0][0]; A[0][1] = MatrixA[0][1]; A[0][2] = MatrixA[0][2];
	A[1][0] = MatrixA[1][0]; A[1][1] = MatrixA[1][1]; A[1][2] = MatrixA[1][2];
	A[2][0] = MatrixA[2][0]; A[2][1] = MatrixA[2][1]; A[2][2] = MatrixA[2][2];
	L.set(vecL[0],vecL[1],vecL[2]); 
	F.set(0.0,0.0,0.0); 
	torque.set(0.0,0.0,0.0);

	normals = new Vector3d[nbr_of_normals];
	for (i = 0; i < nbr_of_normals; i++){
		normals[i].set(norm[i][0],norm[i][1],norm[i][2]);
	}
	vels = new Vector3d[nbr];
	vertices = new Vector3d[nbr];
	for (i = 0; i < nbr; i++){
		vertices[i].set(verts[i][0],verts[i][1],verts[i][2]);
		xcm = xcm + verts[i]/nbr;
	}
	xcmq = Vector3d(xcm);

	radiiq = new Vector3d[nbr];
	radii = new Vector3d[nbr];
	if (invM != 0.0){
		float mi = (1.0/invM)/nbr; 
		for (i = 0; i < nbr; i++){
			radiiq[i] = verts[i] - xcm;
			radii[i] = verts[i] - xcm;
			a += -mi*(-radiiq[i][1]*radiiq[i][1]-radiiq[i][2]*radiiq[i][2]);
			b += -mi*radiiq[i][0]*radiiq[i][1];
			c += -mi*radiiq[i][0]*radiiq[i][2];
			d += -mi*radiiq[i][0]*radiiq[i][1];
			e += -mi*(-radiiq[i][0]*radiiq[i][0]-radiiq[i][2]*radiiq[i][2]);
			f += -mi*radiiq[i][1]*radiiq[i][2];
			g += -mi*radiiq[i][0]*radiiq[i][2];
			h += -mi*radiiq[i][1]*radiiq[i][2];
			j += -mi*(-radiiq[i][0]*radiiq[i][0]-radiiq[i][1]*radiiq[i][1]);
		}

		//det(A) = a11a22a33 + a12a23a31 + a13a21a32 - a13a22a31 - a11a23a32 - a12a21a33
		det = a*e*j + b*f*g + c*d*h - c*e*g - a*f*h - b*d*j;

		Iq[0][0] = 1/det * (e*j-f*h);
		Iq[0][1] = 1/det * (c*h-b*j);
		Iq[0][2] = 1/det * (b*f-c*e);
		Iq[1][0] = 1/det * (f*g-d*j);
		Iq[1][1] = 1/det * (a*j-e*g);
		Iq[1][2] = 1/det * (c*d-a*f);
		Iq[2][0] = 1/det * (d*h-e*g);
		Iq[2][1] = 1/det * (b*g-a*h);
		Iq[2][2] = 1/det * (a*e-b*d);
	
	}else{
		for (i = 0; i < nbr; i++){
			radiiq[i] = verts[i] - xcm;
			radii[i] = verts[i] - xcm;
		}
		Iq[0][0] = 0.0;
		Iq[0][1] = 0.0;
		Iq[0][2] = 0.0;
		Iq[1][0] = 0.0;
		Iq[1][1] = 0.0;
		Iq[1][2] = 0.0;
		Iq[2][0] = 0.0;
		Iq[2][1] = 0.0;
		Iq[2][2] = 0.0;
	}

	//I = A*Iq*A'
	transpose(A,temp1);
	matrix_mult(Iq,temp1,temp2);
	matrix_mult(A,temp2,I);

	//w = I*L
	w.set(Vector3d(I[0]).DotProduct(L),Vector3d(I[1]).DotProduct(L),Vector3d(I[2]).DotProduct(L));
}

RIGID_BODY::~RIGID_BODY(void)
{
}
