#pragma once
#include <math.h>

class Vector3d {
public:
	// Data
	float x, y, z;

	// Ctors	
	Vector3d(float InX, float InY, float InZ) {
		x = InX;
		y = InY;
		z = InZ;
	}
	Vector3d(const Vector3d& V2) {
		x = V2.x;
		y = V2.y;
		z = V2.z;
	}
	Vector3d(float vec[3]) {
		x = vec[0];
		y = vec[1];
		z = vec[2];
	}
	Vector3d() {
		x = 0.0f;
		y = 0.0f;
		z = 0.0f;
	}

	// Operator Overloads
	inline bool operator== (const Vector3d& V2) const 		{
		if (x == V2.x && y == V2.y && z == V2.z) return true;
		else return false;
	}

	inline Vector3d operator+ (const Vector3d& V2) const {
		return Vector3d(x + V2.x,  y + V2.y,  z + V2.z);
	}
	inline Vector3d operator- (const Vector3d& V2) const {
		return Vector3d(x - V2.x,  y - V2.y,  z - V2.z);
	}
	inline Vector3d operator- () const {
		return Vector3d(-x, -y, -z);
	}

	inline Vector3d operator/ (float S) const {
		float fInv = 1.0f / S;
		return Vector3d (x * fInv , y * fInv, z * fInv);
	}
	inline Vector3d operator/ (const Vector3d& V2) const {
		return Vector3d (x / V2.x,  y / V2.y,  z / V2.z);
	}
	inline Vector3d operator* (const Vector3d& V2) const {
		return Vector3d (x * V2.x,  y * V2.y,  z * V2.z);
	}
	inline Vector3d operator* (float S) const {
		return Vector3d (x * S,  y * S,  z * S);
	}

	inline float operator[] (unsigned i) const {
		switch(i) {
			case 0:
				return x;
			case 1:
				return y;
			case 2:
				return z;
			default:
				return 0;
		}
	}

	// Functions
	inline float DotProduct(const Vector3d &V1) {
		return V1.x*x + V1.y*y + V1.z*z;
	}

	inline Vector3d CrossProduct(const Vector3d &V2) {
		return Vector3d(
			y * V2.z  -  z * V2.y,
			z * V2.x  -  x * V2.z,
			x * V2.y  -  y * V2.x);
	}

	inline void set(float xval, float yval, float zval){
		x = xval;
		y = yval;
		z = zval;
	}

	// These require math.h for the sqrtf function
	float magnitude() {
		return sqrtf(x*x + y*y + z*z);
	}

	// Normalize the vector to unit length. Return value is
	// the original length of the vector before normalization.
	inline float normalize() {
 	 	float mag = magnitude();
 
  		if (mag == 0)
  			return mag;
 
  		float mult = 1.0f / mag;
  			x *= mult;
  			y *= mult;
 			z *= mult;
 
  		// return original magnitude
  		return mag;
 }

	float Distance(const Vector3d &V1) {
		return (*this - V1).magnitude();	
	}
};
